/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.ws.client;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPEnvelope;
import org.ow2.petals.ws.addressing.EndpointReference;
import org.ow2.petals.ws.notification.WsnConstants;
import org.ow2.petals.ws.util.AxiomHelper;

/**
 * The subscription client.
 *
 * @author chamerling - eBM WebSourcing
 *
 */
public class SubscriptionClient extends AbstractWSClient {

    /**
     * Creates a new instance of {@link SubscriptionClient}
     *
     * @param sourceEPR
     * @param destinationEPR
     * @param soapClient
     */
    public SubscriptionClient(EndpointReference sourceEPR,
            EndpointReference destinationEPR, SoapClient soapClient) {
        super(sourceEPR, destinationEPR, soapClient);
    }

    /**
     * Creates a new instance of {@link SubscriptionClient}
     *
     * @param sourceEPR
     * @param destinationEPR
     */
    public SubscriptionClient(EndpointReference sourceEPR,
            EndpointReference destinationEPR) {
        super(sourceEPR, destinationEPR);
    }

    /**
     * Unsubscribe
     *
     */
    public void unsubscribe() {
        OMElement body = AxiomHelper
                .createOMElement(WsnConstants.UNSUBSCRIBE_QNAME);
        SOAPEnvelope envelope = callWS(WsnConstants.UNSUBSCRIBE_URI, body);
    }

    /**
     * Pause the subscription
     *
     */
    public void pause() {
        OMElement body = AxiomHelper.createOMElement(WsnConstants.PAUSE_QNAME);
        SOAPEnvelope soapEnv = callWS(WsnConstants.PAUSE_URI, body);
    }

    /**
     * Resume the subscription
     *
     */
    public void resume() {
        OMElement body = AxiomHelper.createOMElement(WsnConstants.RESUME_QNAME);
        SOAPEnvelope soapEnv = callWS(WsnConstants.RESUME_URI, body);
    }

}
