/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.ws.client;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPEnvelope;
import org.ow2.petals.ws.addressing.EndpointReference;
import org.ow2.petals.ws.fault.WsnFault;
import org.ow2.petals.ws.notification.NotificationMessage;
import org.ow2.petals.ws.notification.WsnConstants;
import org.ow2.petals.ws.notification.WsnHelper;
import org.ow2.petals.ws.util.AxiomHelper;

/**
 * This class is used by the provider to send notification to the WSN consumer.
 *
 * @author chamerling - eBM WebSourcing
 *
 */
public class WsnConsumerClient extends AbstractWSClient {

    /**
     * Creates a new instance of {@link WsnConsumerClient}
     *
     * @param sourceEPR
     * @param destinationEPR
     */
    public WsnConsumerClient(EndpointReference sourceEPR,
            EndpointReference destinationEPR) {
        super(sourceEPR, destinationEPR);
    }

    /**
     * Creates a new instance of {@link WsnConsumerClient}
     *
     * @param sourceEPR
     * @param destinationEPR
     * @param soapClient
     */
    public WsnConsumerClient(EndpointReference sourceEPR,
            EndpointReference destinationEPR, SoapClient soapClient) {
        super(sourceEPR, destinationEPR, soapClient);
    }

    /**
     *
     * @return
     */
    public EndpointReference getProducerEPR() {
        return getSourceEPR();
    }

    /**
     *
     * @return
     */
    public EndpointReference getConsumerEPR() {
        return getDestinationEPR();
    }

    /**
     *
     * @param message
     * @return {@link SOAPEnvelope} notification response sent by the
     *         notification consumer or error
     * @throws WsnFault
     */
    public SOAPEnvelope notify(NotificationMessage message) throws WsnFault {
        return notify(new NotificationMessage[] { message });
    }

    /**
     * Notify the notification consumer.
     *
     * @param messages
     * @return {@link SOAPEnvelope} notification response
     * @throws WsnFault
     */
    public SOAPEnvelope notify(NotificationMessage[] messages) throws WsnFault {
        EndpointReference epr = getProducerEPR();

        for (NotificationMessage message : messages) {
            message.setProducerEPR(epr);
        }

        OMElement notifyElement = AxiomHelper
                .createOMElement(WsnConstants.NOTIFY_QNAME);

        for (NotificationMessage message : messages) {
            OMElement messageElement = WsnHelper
                    .notificationMessageToOM(message);
            notifyElement.addChild(messageElement);
        }
        return callWS(WsnConstants.NOTIFY_URI, notifyElement);
    }
}
