/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.ws.client;

import java.util.Date;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPEnvelope;
import org.ow2.petals.ws.addressing.EndpointReference;
import org.ow2.petals.ws.fault.WsnFault;
import org.ow2.petals.ws.notification.Filter;
import org.ow2.petals.ws.notification.GetCurrentMessage;
import org.ow2.petals.ws.notification.GetCurrentMessageResponse;
import org.ow2.petals.ws.notification.NotificationMessage;
import org.ow2.petals.ws.notification.Subscribe;
import org.ow2.petals.ws.notification.WsnConstants;
import org.ow2.petals.ws.notification.WsnHelper;

/**
 * A Ws notification producer client.
 *
 * @author chamerling - eBM WebSourcing
 *
 */
public class WsnProducerClient extends AbstractWSClient {

    /**
     * Creates a new instance of {@link WsnProducerClient}
     *
     * @param sourceEPR
     * @param destinationEPR
     */
    public WsnProducerClient(EndpointReference sourceEPR,
            EndpointReference destinationEPR) {
        super(sourceEPR, destinationEPR);
    }

    /**
     * Creates a new instance of {@link WsnProducerClient}
     *
     * @param sourceEPR
     * @param destinationEPR
     * @param soapClient
     */
    public WsnProducerClient(EndpointReference sourceEPR,
            EndpointReference destinationEPR, SoapClient soapClient) {
        super(sourceEPR, destinationEPR, soapClient);
    }

    /**
     * Subscribe to a topic.
     *
     * @param consumer
     * @param filter
     * @param termination
     * @return a subscription client used to pause or resume the subscription.
     * @throws WsnFault
     */
    public SubscriptionClient subscribe(EndpointReference consumer,
            Filter filter, Date termination) throws WsnFault {
        Subscribe sub = new Subscribe(consumer, filter, termination);

        OMElement subElement = WsnHelper.subscribeToOM(sub);

        SOAPEnvelope responseElement = callWS(WsnConstants.SUBSCRIBE_URI,
                subElement);
        // if (responseElement != null) {
        // System.out.println(responseElement.toString());
        // }

        // TODO : get Subscribe response from responseElement
        // SubscribeResponse response = WsnHelper
        // .subscribeResponseFromOM(responseElement.getBody());
        // EndpointReference epr = response.getSubscriptionReference();

        return new SubscriptionClient(getSourceEPR(), getDestinationEPR());
    }

    /**
     * Get the current message from a given topic.
     *
     * @param topicPath
     * @return
     * @throws WsnFault
     */
    public NotificationMessage getCurrentMessage(QName topicPath)
            throws WsnFault {
        GetCurrentMessage current = new GetCurrentMessage(topicPath);
        OMElement currentElement = WsnHelper.getCurrentMessageToOM(current);

        OMElement responseElement = callWS(WsnConstants.GET_CURRENT_URI,
                currentElement);

        GetCurrentMessageResponse response = WsnHelper
                .getCurrentMessageResponseFromOM(responseElement);
        return response.getCurrentMessage();
    }
}
