/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.ws.notification;

import java.util.Date;

import org.apache.axiom.soap.SOAPEnvelope;
import org.ow2.petals.ws.addressing.EndpointReference;
import org.ow2.petals.ws.client.SimpleSoapClient;
import org.ow2.petals.ws.client.SoapClient;
import org.ow2.petals.ws.client.WsnConsumerClient;
import org.ow2.petals.ws.fault.WsnFault;

/**
 * Class used to manage subscription. It makes the link between the notification
 * consumer and provider.
 *
 * @author chamerling - eBM WebSourcing
 *
 */
public class Subscription {

    /**
     * The client used to send messages to the client
     */
    protected WsnConsumerClient client;

    /**
     * The EPR of the notification consumer
     */
    protected EndpointReference consumerEPR;

    /**
     * The creation time of the current subscription
     */
    protected Date creationTime;

    /**
     * The associated filter of the subscription
     */
    protected Filter filter;

    /**
     * The EPR of the notification producer
     */
    protected EndpointReference producerEPR;

    /**
     * Creates a new instance of {@link Subscription}
     *
     */
    public Subscription() {
        this.creationTime = new Date();
    }

    /**
     *
     * @param soapClient
     * @return
     */
    protected WsnConsumerClient createConsumerClient(SoapClient soapClient) {
        EndpointReference consumer = getConsumerEPR();
        EndpointReference producer = getProducerEPR();

        if (soapClient == null) {
            soapClient = new SimpleSoapClient();
        }
        WsnConsumerClient client = new WsnConsumerClient(producer, consumer,
                soapClient);
        return client;
    }

    /**
     *
     * @return
     */
    protected synchronized WsnConsumerClient getConsumerClient() {
        if (this.client == null) {
            this.client = createConsumerClient(null);
        }
        return this.client;
    }

    /**
     *
     * @param soapClient
     * @return
     */
    protected synchronized WsnConsumerClient getConsumerClient(
            SoapClient soapClient) {
        if (this.client == null) {
            this.client = createConsumerClient(soapClient);
        }
        return this.client;
    }

    /**
     *
     * @return
     */
    public EndpointReference getConsumerEPR() {
        return consumerEPR;
    }

    public Date getCreationTime() {
        return creationTime;
    }

    public Filter getFilter() {
        return this.filter;
    }

    public EndpointReference getProducerEPR() {
        return this.producerEPR;
    }

    /**
     *
     * @param consumer
     */
    public void setConsumerEPR(EndpointReference consumer) {
        if (consumer == null) {
            throw new NullPointerException("");
        }
        this.consumerEPR = consumer;
    }

    /**
     *
     * @param filter
     */
    public void setFilter(Filter filter) {
        this.filter = filter;
    }

    /**
     *
     * @param producerEPR
     */
    public void setProducerEPR(EndpointReference producerEPR) {
        if (producerEPR == null) {
            throw new NullPointerException("");
        }
        this.producerEPR = producerEPR;
    }

    /**
     * Publish a notification message
     *
     * @param message
     */
    public void publish(NotificationMessage message) {
        this.publish(message, null);
    }

    /**
     *
     * @param message
     * @param soapClient
     */
    public void publish(NotificationMessage message, SoapClient soapClient) {
        if (!getFilter().accept(message)) {
            return;
        }

        message.setProducerEPR(getProducerEPR());
        message.setSubscriptionEPR(getConsumerEPR());

        try {
            WsnConsumerClient client = getConsumerClient(soapClient);
            SOAPEnvelope envelope = client.notify(message);
        } catch (WsnFault e) {
            e.printStackTrace();
        }
    }
}
